( 3iSynth.fs - AE - 2020-12-05---Sat---08:21 )
( This is the development file for 76477 sound generation chip with just 3iForth.avr )
( Microcontroller is convenient for software coordination of complex inputs )
( Combine with diagnostic lights )

( It uses 2x Nano controllers, one for the Piano to read the keys and play the tones, )
(    and one to control the synth chip and its modes, and things like drum beat/percussion timings  )
( Could try to combine into one, including building in the loops in .avr )

\ SOUND ENABLE/ONE-SHOT TRIGGER
2 constant p9  \ green
p9 constant trig
p9 constant sound
: sound-inhibit p9 hi ;  \ connect RED led to ground via resistor
: sound-allow p9 lo ;    \ connect GREEN led to VCC via resistor
: sound-shot sound-inhibit 100 ms sound-allow ;

\ VCO SELECT
3 constant p22  \ white
: vco-internal p22 hi ;
: vco-external p22 lo ;

\ DUTY CYCLE (LOUD/SOFT)
18 constant p19 \ white
: duty-loud  p19 lo ;
: duty-soft  p19 hi ;

\ ENVELOPE SELECT
4 constant p1   \ red
5 constant p28  \ brown
: env-vco     sound-inhibit p1 lo p28 lo sound-allow ;
: env-vco-alt sound-inhibit p1 hi p28 hi sound-allow ;
: env-mixer   sound-inhibit p1 lo p28 hi sound-allow ;
: env-oneshot sound-inhibit p1 hi p28 lo sound-allow ;

\ MIXER SELECT
9 constant pXX \ notinuse
6 constant p27 \ mixC
7 constant p25 \ mixB
8 constant p26 \ mixA  lsb
: set-mixer ( 0-7 --- ) sound-inhibit port4write sound-allow ;
: mix-vco           0 set-mixer ;
: mix-slf           1 set-mixer ;
: mix-noise         2 set-mixer ;
: mix-vco-noise     3 set-mixer ;
: mix-slf-noise     4 set-mixer ;
: mix-vco-slf-noise 5 set-mixer ;
: mix-vco-slf       6 set-mixer ;
: mix-inhibit       7 set-mixer ;

\ POWER TO CHIP
17 constant p15  \ purple - power/reset  5mA for the chip itself
p15 constant power

: synth-sleep power lo ;
: synth-wake  power hi ;


: synth-setup
  power out 100 ms  
  power hi 100 ms
  p9 out 100 ms
  p22 out 100 ms
  p19 out 100 ms
  p1 out 100 ms
  p28 out 100 ms
  pXX p27 p25 p26 ( lsb ) port4out 100 ms  \ 3-bit port for mixer select
  sound-inhibit
;

: piano-setup
  vco-external
  duty-loud
  mix-vco
  sound-allow
;

: piano-noise
  vco-external
  duty-loud
  mix-vco-slf-noise
  sound-allow
;

: laser-setup
    env-vco
    vco-internal
    mix-vco 
    sound-inhibit ;
: fire sound-allow power lo 10 ms power hi ;  \ restarts the modulation -- unplug green SLF p20 resistor
: explode sound-allow mix-noise 2000 ms sound-inhibit ;


\ 74150  16-to-1 multiplex IC
\ CODE REVIEW: replace IC with 74151 8-to-1 multiplex chip
10 constant mux-lsb  \ white
11 constant mux-pin2 \ green
12 constant mux-pin3 \ tan
14 constant mux-msb  \ grey

15 constant mux-input \ yellow 
16 constant mux-enable \ green 


: 74150-setup  
    mux-input pullup 50 ms  \ chip output signal that will be read
    mux-enable out 50 ms     \ chip strobe
    mux-enable lo           \      enabled     
    14 12 11 10 ( lsb ) port4Cout 100 ms  \ 4-bit port C for selecting channel for the multiplex line in chip 74150
;

: test 15 poll-pin ;
: 74150-test 7 0 do i dup . .s ." : " port4Cwrite test cr ." next " loop ;


\ laser
: drop-bomb mix-vco fire 4000 ms explode ;
: fire-adam mix-vco-noise fire 4000 ms explode ;
: siren-explode mix-vco fire sound-allow 4000 ms mix-vco-noise explode ;

\ For Use with C21d (short burst)
: laser-loop mix-vco begin fire 100 ms fire 200 ms key? until ; 
: alarm-loop mix-vco begin fire 200 ms key? until ;

\ For Use with C21c (medium burst) - firecrackers
: laser2-loop begin fire 500 ms fire 2000 ms key? until ; 
: alarm2-loop begin fire 2000 ms key? until ;

\ For Use with C21c (medium burst) - falling bombs
: laser3-loop begin fire 1500 ms fire 5000 ms key? until ; 
: alarm3-loop begin fire 10000 ms key? until ;

: noisy-laser-setup
   env-mixer
   vco-internal
   mix-vco-noise ;
\ fire

: noise-setup
    env-oneshot
    vco-internal
    mix-noise
    trig hi ; \ suppress
: noise ( ms -- ) trig lo ms trig hi ;
: crackle  150 noise 250 ms 300 noise 400 ms 400 noise ;  \ walkie talkie

: drum-setup
    env-oneshot
    mix-noise   \ also mix-vco-noise combines tone with rhythm (like tambourine)
    vco-external
    sound-allow ;
: tambourine-setup  \ set R5 midway between pure tone and noise for the hit and bell ring
    env-oneshot
    vco-internal
    mix-vco-noise
    sound-allow ;
: gunshot-setup  drum-setup ;
: siren-setup
    env-oneshot
    vco-internal
    mix-vco
    sound-allow ;

\ PLANE
: prop-plane
  env-mixer 100 ms
  mix-vco-noise 100 ms
  sound-allow 100 ms ;

\ BATTLE SEQUENCE
: shot p9 hi p9 lo ;
: burst 12 0 do shot 40 ms loop p9 hi ;
: volley 12 0 do shot 70 ms loop p9 hi ;
: two-shots 2 0 do shot 330 ms loop ;
: firing volley 270 ms two-shots 100 ms burst 80 ms volley ;
: semiauto 10 0 do shot 90 ms loop ;
: brace 2 0 do shot 150 ms loop ;
: exchange brace 20 ms shot 150 ms two-shots 33 ms 100 ms two-shots 53 ms shot 730 ms brace ;
: battle exchange 200 ms volley 20 ms burst 140 ms firing 80 ms semiauto 20 ms brace 100 ms two-shots 300 ms two-shots 120 ms shot 30 ms semiauto ;

\ DRUM PAD  
\ 1-e-+-a  16th notes.  1 beat = 16th note, 2 = 8th note, 4 = quarter note
: beat 100 ms ;
: beats ( n -- ) 0 do beat loop ;
: snare shot 4 beats shot 4 beats shot 2 beats shot beat shot 4 beats beat ;
: snare-loop begin snare key? until ;

: snare1 
    shot 
    4 beats shot 
    4 beats shot 
    2 beats shot 
    1 beats shot 
    4 beats beat ;
: snare1-loop begin snare key? until ;

: snare2  \ all 16th notes
    shot 
    1 beats shot 
    1 beats shot 
    1 beats shot ;

: snare3
    shot 
    2 beats shot 
    2 beats shot 
    2 beats shot 
       beat shot 
    4 beats beat ;
: snare3-loop begin snare1 snare3 key? until ;  \ 1 2 3-+-a 4 + 5 + a   3/3 measure

: metronome 
    shot 
    4 beats ;
: metronome-loop begin metronome key? until ;


\ TODO:
\ ** ADD DRUM BEAT SAMPLER - 4 bar using 16 DIP switches and 74150 multiplexer to loop through the selection




\ -----------------------------------------------

: help-synth
  cr
  cr ." HELP-SYNTH"
  cr ." ----------"
  cr ." Setup: "
  cr ." HW/Circuit: Programmable Digital Piano with Sn76477 chip, wired per schematic notes."
  cr ." Target uC: Arduino Nano, programmed with 3iForth.avr"
  cr ." Host PC: Realterm (FF FE). GForth (include 3iForth.fs, include 3iSynth.fs)"
  cr ." Instructions:"
  cr ."    1 open-com   rx"
  cr ."    synth-setup"
  cr ."        duty-loud duty-soft synth-sleep synth-wake sound-inhibit sound-allow"
  cr ."    drum-setup snare3-loop metronome-loop"
  cr ."    tambourine-setup "
  cr ."    gunshot-setup battle"
  cr ."    laser-setup fire sound-inhibit sound-allow"
  cr ."    noise-setup crackle"
  cr ."    noisy-laser-setup fire"
  cr ."    noisy-laser-setup fire"
  cr ."    prop-plane"
  cr
  cr ."  74150-setup"
  cr ."  74150-test"
;


help-synth

\ VERSION 1: 2020-12-07---Mon---00:54 - Totally cool!  Have now hooked 76477 up and we can get tones AND drum beat effects on the same speaker for 36mA max and its fairly loud! :)
\ 2020-12-11---Fri---07:16 - Exploring capabilities, saving settings and building in uC
\ VERSION 2: 2020-12-15---Tue---02:49  Incorporating the 76477 complex sound generator chip effects with the Piano tones! proto wired board
\ 2020-12-20---Sun---10:23 --- added control boards (VCO, SLF, NOISE, ONE-SHOT ATTACK/DECAY, VOLUME/AMPLITUDE) and 74150 (should be 74151) N-to-1 multiplexer chip to strobe through 7 binary inputs --- 4 envelope select modes (xor), and 3 mixer select modes (or, combinable)
\ VERSION 3: 2020-12-21---Mon---08:26 -- got multiplex chip working --- now need to integrate user selections into code response


